#appModules/Dragon naturally speaking
import appModuleHandler
import addonHandler
addonHandler.initTranslation()
from logHandler import log
import winUser
import controlTypes
import os
import gui
import wx
import ui
import speech
import api
from NVDAObjects.window import Window
from NVDAObjects.IAccessible import IAccessible, getNVDAObjectFromEvent
import time
import eventHandler
from natspeak_docHandler import openDocPath
from ctypes import windll
import speech
import oleacc
import config

user32 = windll.user32

# nvda 2014.3 addonHandler adds new help button
# so docHandler  is no more necessary 
_newAddonHelpButton = hasattr(addonHandler.Addon, "getDocFilePath")

_addonDir = os.path.join(os.path.dirname(__file__), "..").decode("mbcs") # The root of an addon folder
_curAddon = addonHandler.Addon(_addonDir) # Addon instance
_addonSummary = _curAddon.manifest['summary']
_addonVersion = _curAddon.manifest['version']
_addonName = _curAddon.manifest['name']
_scriptCategory = unicode(_addonSummary)
# a global spellIt task timer
_spellItTaskTimer = None
#last  information box text save
_lastInformationBoxText = ""
# constants
# for winUser getWindow
GW_HWNDFIRSTCHILD = 5
		
def spellIt(text):
	global _spellItTaskTimer
	def callback (text):
		global _spellItTaskTimer
		_spellItTaskTimer = None
		speech.speakSpelling(text)
		
	if _spellItTaskTimer != None:
		_spellItTaskTimer.Stop()
	
	_spellItTaskTimer = wx.CallLater(1000, callback, text)

	
	
def stopSpellIt():
	global _spellItTaskTimer
	if _spellItTaskTimer:
		_spellItTaskTimer.Stop()
		_spellItTaskTimer = None



class InSpellWindow(Window):

	
	def initOverlayClass(self):
		self.bindGesture("kb:enter", "enterKey")
		self.bindGesture("kb:space", "enterKey")
		
	def script_enterKey(self, gesture):
		if self.role == controlTypes.ROLE_BUTTON and _("Replay") in self.name:
			self.doAction()
			time.sleep(3.0)
			self.setFocus()
			eventHandler.queueEvent("gainFocus",self)
		else:
			gesture.send()


		


#	def event_nameChange(self):
		#super(InSpellList,self). Event_nameChange()

	def _get_name(self):
		name = super(InSpellWindow,self)._get_name()
		if self.role == controlTypes.ROLE_EDITABLETEXT:
			name =_("Spell or write what you said")
		return name
		
		
	def event_valueChange(self):
		super(InSpellWindow,self).event_valueChange()
		stopSpellIt()
		if self.role == controlTypes.ROLE_EDITABLETEXT:
			spellIt(self.value)



	def event_gainFocus(self):
		super(InSpellWindow, self).event_gainFocus()
		stopSpellIt()
		if self.role == controlTypes.ROLE_EDITABLETEXT:
			spellIt(self.value)
		elif self.role == controlTypes.ROLE_LISTITEM:
			spellIt(self.name)

		
			



class InDayHelpWindow(Window):
	def initOverlayClass(self):
		self.bindGesture("kb:alt+control+1", "sayTypeOfTheDay")
	
		
	def script_sayTypeOfTheDay(self, gesture):
		ui.message( self.appModule.getTypeOfTheDay(api.getForegroundObject()))


	script_sayTypeOfTheDay.__doc__ = _("Report the type of the day")
	script_sayTypeOfTheDay.category = _scriptCategory
		
		
class AppModule(appModuleHandler.AppModule):
	scriptCategory = _scriptCategory
	#allowed versions for this module
	_allowedVersions = [ "10","11", "12"]
	# wwindow class and name
	_barMainWindow = None
	_spellWindow = None
	_resultsBoxWindow = None
	_typeOfTheDayWindow = None
	_informationBoxChildID = None
	
	_dragonSpeakingTimer = None
	_lastSpokenText = ""
	
	
	def checkCompatibility(self):
		majorVersion= self.productVersion.split(".")[0]
		if majorVersion not in self._allowedVersions:
			message = _("Tise installed add-on has not been validated with this   version of Dragon Naturally Speaking and may not behave as expected") 
			title = _("Warning")
			wx.CallLater(5000,gui.messageBox,message, title)
			
	def initWindowClassAndName(self):
		self._barMainWindow = ("DgnBarMainWindowCls", None)
		self._typeOfTheDayWindow = ("#32770", u"Conseil du jour".encode("mbcs"))
		if self.isMajorVersion("10"):
			self._spellWindow = ("#32770", _("Spell").encode("mbcs"))
			self._resultsBoxWindow = ("#32770", None)
			self._informationBoxChildID = 6
			
		else:
			self._spellWindow = ("ClsSpellDialog", _("Spell window").encode("mbcs"))
			self._resultsBoxWindow = ("DgnResultsBoxWindow",  None)
			self._informationBoxChildID = 5


	def __init__(self, *args, **kwargs):
		super(AppModule, self).__init__(*args, **kwargs)
		# turn off speakTypCharacter flag  during dragon application
		self.speakTypeCharacter = config.conf["keyboard"]["speakTypedCharacters"]
		config.conf["keyboard"]["speakTypedCharacters"]=False
		
		self.checkCompatibility()
		self.initWindowClassAndName()
		self.startDragonSpeakingTimer()
		if not _newAddonHelpButton:		
			# set help menu
			self.help = gui.mainFrame.sysTrayIcon.helpMenu
			self.helpItem = self.help.Append(wx.ID_ANY, u"{summary} {version}".format(summary=_addonSummary, version=_addonVersion), _addonName)
			gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onHelp, self.helpItem)
		
	def onHelp(self, evt):
		openDocPath()
			
	def terminate(self):
		# restore speakTypeCharacter flag
		config.conf["keyboard"]["speakTypedCharacters"]=self.speakTypeCharacter
		self.stopDragonSpeakingTimer()

		try:
			if not _newAddonHelpButton:
				self.help.RemoveItem(self.helpItem)
		except wx.PyDeadObjectError:
			pass
			
	def chooseNVDAObjectOverlayClasses(self, obj, clsList):
		h = winUser.getForegroundWindow()
		c = winUser.getClassName(h)
		n = winUser.getWindowText(h).encode("mbcs")
		if (c,n) == self._typeOfTheDayWindow:
			clsList.insert(0, InDayHelpWindow)
		elif (c,n)  == self._spellWindow:
			clsList.insert(0, InSpellWindow)



		
		
	def event_appModule_loseFocus(self):
		if hasattr(self, "sayTypeOfTheDayMonitor") and self.sayTypeOfTheDayMonitor:
			self.sayTypeOfTheDayMonitor.Stop()
			self.sayTypeOfTheDayMonitor = None

		
		
		
		
	def getTypeOfTheDay(self, oForeground):
		if not self.isMajorVersion("12"):
			# dragon 10 and 11
			if oForeground.childCount:
				return oForeground.children[2].name
			else:
				return ""
		
		#dragon 12
		text = ""
		h =user32.FindWindowExA(winUser.getDesktopWindow(), None,"#32770","Conseil du jour")
		h = user32.FindWindowExA(h, None, "AfxFrameOrView100u",None)
		h = user32.FindWindowExA(h, None, "Shell Embedding", None)
		h = user32.FindWindowExA(h, None, "Shell DocObject View", None)
		if h :
			obj =getNVDAObjectFromEvent(h,-4,0)
			obj = obj.children[0].children[0]
			for i in range (1,obj.childCount) :
				o = obj.children[i]
				name = o.name
				if (o.role==controlTypes.ROLE_STATICTEXT and not controlTypes.STATE_INVISIBLE in o.states 
					and not name in ("C","onseils") ):
					text = text +" %s" % name

		return text

	def sayTypeOfTheDay(self):
		oForeground = api.getForegroundObject()
		if (oForeground.windowClassName, oForeground.name) != self._typeOfTheDayWindow:
			self.sayTypeOfTheDayMonitor = None
			return
			
		typeOfTheDay = self.getTypeOfTheDay(oForeground)
		if typeOfTheDay != self.typeOfTheDay:
			ui.message(typeOfTheDay)
			self.typeOfTheDay = typeOfTheDay

		
		self.sayTypeOfTheDayMonitor = wx.CallLater(1000, self.sayTypeOfTheDay)
	
	def event_foreground(self,obj, nextHandler):
		
		if (obj.windowClassName, obj.name) == self._typeOfTheDayWindow :
			if hasattr(self, "sayTypeOfTheDayMonitor") and  self.sayTypeOfTheDayMonitor != None:
				self.sayTypeOfTheDayMonitor.Stop()

			self.typeOfTheDay = self.getTypeOfTheDay(obj)
			self.sayTypeOfTheDayMonitor = wx.CallLater(500, self.sayTypeOfTheDay)

		else:
			if hasattr(self, "sayTypeOfTheDayMonitor") and  self.sayTypeOfTheDayMonitor != None:
				self.sayTypeOfTheDayMonitor.Stop()
				self.sayTypeOfTheDayMonitor = None


				
	def startDragonSpeakingTimer(self):
		# stop it if is running
		self.stopDragonSpeakingTimer()
		#now start it
		self._dragonSpeakingTimer = wx.CallLater(1000, self.readResult)
		
	def stopDragonSpeakingTimer(self):
		if self._dragonSpeakingTimer:
			self._dragonSpeakingTimer.Stop()
			self._dragonSpeakingTimer = None
			
	def  isMajorVersion(self, majorVersion):
		return self.productVersion.split(".")[0] == majorVersion
	

	def getFilteredText(self,text):

		dragonFilterDic = {
			_("acoustic"): "",
			_("Text field unknown"): "",
			_("Normal mode"):_("Normal mode"),
			"Mode dict": _("Dictation mode"),
			_("Commands mode"): _("Commands mode"),
			_("The Dragon's microphone is turned off") : _("The microphone is turned off"),
			_("The microphone is in standby"): _("The microphone is in standby")
			}
		newText = text
		for t in dragonFilterDic:
			if t in text:
				newText = dragonFilterDic[t]
				break
		
		return newText

	
	def reportInformationBox(self):
		global _lastInformationBoxText
		(c,n) = self._barMainWindow
		h= user32.FindWindowExA(winUser.getDesktopWindow(), None, c, n)
		if not h:
			return
		
		h = winUser.getWindow(h, GW_HWNDFIRSTCHILD )
		# to find information box
		i = self._informationBoxChildID
		while i:
			h = winUser.getWindow(h, winUser.GW_HWNDNEXT)
			i= i-1
		
		text = self.getFilteredText(winUser.getWindowText(h))
		if text != "" and text != _lastInformationBoxText:
			_lastInformationBoxText = text
			ui.message(text)

	
	def readResult(self):
		self.reportInformationBox()
		(c,n) = self._resultsBoxWindow
		h = user32.FindWindowExA(winUser.getDesktopWindow(), None, c, n)
		if not h or not winUser.isWindowVisible(h):
			#no result but restart timer
			self.startDragonSpeakingTimer()
			return

		text = winUser.getWindowText(h)
		if text != self._lastSpokenText:
			if text != "<???>" and "..." not in text:
				self._lastSpokenText = text
				speech.speakMessage(text)

		# restart timer
		self.startDragonSpeakingTimer()

			
