import addonHandler
import appModuleHandler
addonHandler.initTranslation()
import globalPluginHandler
import api
import NVDAObjects
import os
import winUser
from oleacc import AccessibleObjectFromWindow
import scriptHandler
import ui
import speech
from ctypes import windll
import time
import eventHandler
# constants
# for winUser getWidow
GW_HWNDFIRSTCHILD = 5


user32 = windll.user32

_addonDir = os.path.join(os.path.dirname(__file__), "..").decode("mbcs") # The root of an addon folder
_curAddon = addonHandler.Addon(_addonDir) # Addon instance
_addonSummary = _curAddon.manifest['summary']
_addonVersion = _curAddon.manifest['version']
_addonName = _curAddon.manifest['name']
_scriptCategory = unicode(_addonSummary)

class GlobalPlugin(globalPluginHandler.GlobalPlugin):
	scriptCategory = _scriptCategory
	# wwindow class and name
	_resultsBoxWindow = None
	_microphoneButton = None
	
	def __init__(self):
		super(GlobalPlugin, self).__init__()
		self.initWindowClassAndName()		
		
	def getDragonProductVersion(self):
		(c,n) = ("DgnBarMainWindowCls", None)
		h = user32.FindWindowExA(winUser.getDesktopWindow(), None, c,n )
		if not h:
			return None
		obj = NVDAObjects.IAccessible.getNVDAObjectFromEvent(h,-4,0)
		appModule = appModuleHandler.AppModule(obj.processID, "")
		if appModule:
			return appModule.productVersion
		return None
			

	def isMajorVersion(self, majorVersion):
		version = self.getDragonProductVersion()
		if version!= None:
			return version.split(".")[0] == majorVersion
		return False
		
	def isDragonToolBarAlive(self):
		(c,n) = ("DgnBarMainWindowCls", None)
		h= user32.FindWindowExA(winUser.getDesktopWindow(), None, c, n)
		if h:
			return True
		
		return False


	def initWindowClassAndName(self):
		if self.isMajorVersion("10"):
			self._resultsBoxWindow = ("#32770", None)
			self._microphoneButton = 9
			self._informationBoxChildID = 6
			
		else:
			self._resultsBoxWindow = ("DgnResultsBoxWindow",  None)
			self._microphoneButton = 1
			self._informationBoxChildID = 5


	def script_readResultsBoxWindow (self, gesture):
		if not self.isDragonToolBarAlive():
			ui.message(_("Dragon Naturally Speaking is not running"))
			return
			


		self.initWindowClassAndName()
		(c,n) = self._resultsBoxWindow
		h = user32.FindWindowExA(winUser.getDesktopWindow(), None, c, n)
		if not h or not winUser.isWindowVisible(h):
			ui.message(_("The result window is not visible"))
			return


		text = winUser.getWindowText(h)
		if text == "<???>" :
			ui.message(_("No vocal reconition"))
		elif "..." in text:
			ui.message(_("Vocal recognition running"))
		else:
			if scriptHandler.getLastScriptRepeatCount() == 0:
				speech.speakMessage(text)
			else:
				speech.speakSpelling(text)
			return
			
	# Documentation
	script_readResultsBoxWindow.__doc__ = _("say the popup window text that appears during vocal reconition by Dragon Naturally speaking")
	
	def script_microOnOff(self, gesture):
		(c,n) = ("DgnBarMainWindowCls", None)
		h= user32.FindWindowExA(winUser.getDesktopWindow(), None, c, n)
		if not h:
			ui.message(_("Dragon Naturally Speaking is not running"))
			return

		self.initWindowClassAndName()
		h = winUser.getWindow(h, GW_HWNDFIRSTCHILD )
		# to find microphone button
		i = self._microphoneButton
		while i:
			h = winUser.getWindow(h, winUser.GW_HWNDNEXT)
			i= i-1

	# now press microphone button without moving focus
		obj = NVDAObjects.IAccessible.getNVDAObjectFromEvent(h,-4,0)
		oldSpeechMode = speech.speechMode
		oFocus = api.getFocusObject()
		speech.speechMode = speech.speechMode_off
		obj.IAccessibleObject.accDoDefaultAction (0)
		time.sleep(1.0)
		oFocus.setFocus()
		speech.speechMode = oldSpeechMode
		
	script_microOnOff.__doc__ = _("Turn on or off the microphone ")

	def getFilteredText(self,text):
		dragonFilterDic = {
			_("acoustic"): "",
			_("Text field unknown"): "",
			_("Normal mode"):_("Normal mode"),
			"Mode dict": _("Dictation mode"),
			_("Commands mode"): _("Commands mode"),
			_("The Dragon's microphone is turned off") : _("The microphone is turned off"),
			_("The microphone is in standby"): _("The microphone is in standby")
			}
		newText = text
		for t in dragonFilterDic:
			if t in text:
				newText = dragonFilterDic[t]
				break
		
		return newText

		
	def script_reportInformationBox(self, gesture):
		(c,n) = ("DgnBarMainWindowCls", None)
		h= user32.FindWindowExA(winUser.getDesktopWindow(), None, c, n)
		if not h:
			return
		
		h = winUser.getWindow(h, GW_HWNDFIRSTCHILD )
		# to find information box
		i = self._informationBoxChildID
		while i:
			h = winUser.getWindow(h, winUser.GW_HWNDNEXT)
			i= i-1
	
		text = winUser.getWindowText(h)
		ui.message(self.getFilteredText(text))

	script_reportInformationBox.__doc__ = _("Report information box")
	

		
	__gestures = {
		"kb(desktop):control+numPadPlus": "microOnOff",
		"kb(laptop):control+=": "microOnOff",

		"kb:nvda+shift+!":"readResultsBoxWindow",
		"kb:nvda+shift+:":"reportInformationBox"
		}

